@@ -13,7 +13,7 @@ class Circle(Primitive):
 
     Parameters
     ----------
-    plane : :class:`compas.geometry.Plane` or tuple of point and normal
+    plane : [point, vector] | :class:`compas.geometry.Plane`
         The plane of the circle.
     radius : float
         The radius of the circle.
@@ -23,17 +23,17 @@ class Circle(Primitive):
     plane : :class:`compas.geometry.Plane`
         The plane of the circle.
     radius : float
-        The radius.
+        The radius of the circle.
     center : :class:`compas.geometry.Point`
-        The base point of the plane and center of the circle.
-    normal : :class:`compas.geometry.Vector`
-        The normal vector of the plane.
+        The center of the circle.
+    normal : :class:`compas.geometry.Vector`, read-only
+        The normal of the circle.
     diameter : float, read-only
         The diameter of the circle.
-    circumference : float, read-only
-        The circumference of the circle.
     area : float, read-only
         The area of the circle.
+    circumference : float, read-only
+        The circumference of the circle.
 
     Examples
     --------
@@ -41,12 +41,25 @@ class Circle(Primitive):
     >>> from compas.geometry import Circle
     >>> plane = Plane([0, 0, 0], [0, 0, 1])
     >>> circle = Circle(plane, 5)
+
     """
 
     __slots__ = ['_plane', '_radius']
 
+    def __init__(self, plane, radius, **kwargs):
+        super(Circle, self).__init__(**kwargs)
+        self._plane = None
+        self._radius = None
+        self.plane = plane
+        self.radius = radius
+
+    # ==========================================================================
+    # data
+    # ==========================================================================
+
     @property
     def DATASCHEMA(self):
+        """:class:`schema.Schema` : Schema of the data representation."""
         import schema
         return schema.Schema({
             'plane': Plane.DATASCHEMA.fget(None),
@@ -55,15 +68,9 @@ class Circle(Primitive):
 
     @property
     def JSONSCHEMANAME(self):
+        """str : Name of the schema of the data representation in JSON format."""
         return 'circle'
 
-    def __init__(self, plane, radius, **kwargs):
-        super(Circle, self).__init__(**kwargs)
-        self._plane = None
-        self._radius = None
-        self.plane = plane
-        self.radius = radius
-
     @property
     def data(self):
         """dict : The data dictionary that represents the circle."""
@@ -74,9 +81,35 @@ class Circle(Primitive):
         self.plane = Plane.from_data(data['plane'])
         self.radius = data['radius']
 
+    @classmethod
+    def from_data(cls, data):
+        """Construct a circle from its data representation.
+
+        Parameters
+        ----------
+        data : dict
+            The data dictionary.
+
+        Returns
+        -------
+        :class:`compas.geometry.Circle`
+            The constructed circle.
+
+        Examples
+        --------
+        >>> from compas.geometry import Circle
+        >>> data = {'plane': {'point': [0.0, 0.0, 0.0], 'normal': [0.0, 0.0, 1.0]}, 'radius': 5.}
+        >>> circle = Circle.from_data(data)
+
+        """
+        return cls(Plane.from_data(data['plane']), data['radius'])
+
+    # ==========================================================================
+    # properties
+    # ==========================================================================
+
     @property
     def plane(self):
-        """:class:`compas.geometry.Plane` : The plane of the circle."""
         return self._plane
 
     @plane.setter
@@ -85,7 +118,6 @@ class Circle(Primitive):
 
     @property
     def radius(self):
-        """float : The radius of the circle."""
         return self._radius
 
     @radius.setter
@@ -94,17 +126,14 @@ class Circle(Primitive):
 
     @property
     def normal(self):
-        """:class:`compas.geometry.Vector` : The normal of the circle."""
         return self.plane.normal
 
     @property
     def diameter(self):
-        """float: The diameter of the circle."""
         return self.radius * 2
 
     @property
     def center(self):
-        """:class:`compas.geometry.Point` : The center of the circle."""
         return self.plane.point
 
     @center.setter
@@ -113,12 +142,10 @@ class Circle(Primitive):
 
     @property
     def area(self):
-        """float  : The area of the circle."""
         return pi * (self.radius**2)
 
     @property
     def circumference(self):
-        """float : The circumference of the circle."""
         return 2 * pi * self.radius
 
     # ==========================================================================
@@ -162,28 +189,6 @@ class Circle(Primitive):
     # constructors
     # ==========================================================================
 
-    @classmethod
-    def from_data(cls, data):
-        """Construct a circle from its data representation.
-
-        Parameters
-        ----------
-        data : dict
-            The data dictionary.
-
-        Returns
-        -------
-        :class:`compas.geometry.Circle`
-            The constructed circle.
-
-        Examples
-        --------
-        >>> from compas.geometry import Circle
-        >>> data = {'plane': {'point': [0.0, 0.0, 0.0], 'normal': [0.0, 0.0, 1.0]}, 'radius': 5.}
-        >>> circle = Circle.from_data(data)
-        """
-        return cls(Plane.from_data(data['plane']), data['radius'])
-
     # ==========================================================================
     # methods
     # ==========================================================================
@@ -193,9 +198,13 @@ class Circle(Primitive):
 
         Parameters
         ----------
-        T : :class:`compas.geometry.Transformation` or list of list
+        T : :class:`compas.geometry.Transformation` | list[list[float]]
             The transformation.
 
+        Returns
+        -------
+        None
+
         Examples
         --------
         >>> from compas.geometry import Frame
@@ -206,5 +215,6 @@ class Circle(Primitive):
         >>> frame = Frame([1, 1, 1], [0.68, 0.68, 0.27], [-0.67, 0.73, -0.15])
         >>> T = Transformation.from_frame(frame)
         >>> circle.transform(T)
+
         """
         self.plane.transform(T)
