@@ -13,34 +13,31 @@ from compas.geometry.primitives import Point
 from compas.utilities import pairwise
 
 
-__all__ = ['Polyline']
-
-
 class Polyline(Primitive):
-    """A polyline is a sequence of points connected by line segments.
-
-    A polyline is a piecewise linear element.
-    It does not have an interior.
-    It can be open or closed.
-    It can be self-intersecting.
+    """A polyline is defined by a sequence of points connected by line segments.
 
     Parameters
     ----------
-    points : list of point
+    points : list[[float, float, float] | :class:`compas.geometry.Point`]
         An ordered list of points.
         Each consecutive pair of points forms a segment of the polyline.
 
     Attributes
     ----------
-    data : dict
-        The data representation of the polyline.
-    points : list of :class:`compas.geometry.Point`
-        The polyline points.
-    lines : list of :class:`compas.geometry.Line`, read-only
-        The polyline segments.
+    points : list[:class:`compas.geometry.Point`]
+        The points of the polyline.
+    lines : list[:class:`compas.geometry.Line`], read-only
+        The lines of the polyline.
     length : float, read-only
         The length of the polyline.
 
+    Notes
+    -----
+    A polyline is a piecewise linear element.
+    It does not have an interior.
+    It can be open or closed.
+    It can be self-intersecting.
+
     Examples
     --------
     >>> polyline = Polyline([[0,0,0], [1,0,0], [2,0,0], [3,0,0]])
@@ -56,34 +53,73 @@ class Polyline(Primitive):
     True
     >>> polyline.lines[0].length
     1.0
+
     """
 
-    __slots__ = ["_points", "_lines"]
+    __slots__ = ['_points', '_lines']
 
-    def __init__(self, points):
-        super(Polyline, self).__init__()
+    def __init__(self, points, **kwargs):
+        super(Polyline, self).__init__(**kwargs)
         self._points = []
         self._lines = []
         self.points = points
 
+    # ==========================================================================
+    # data
+    # ==========================================================================
+
+    @property
+    def DATASCHEMA(self):
+        """:class:`schema.Schema` : Schema of the data representation."""
+        from schema import Schema
+        from compas.data import is_float3
+        return Schema({
+            'points': lambda points: all(is_float3(point) for point in points)
+        })
+
+    @property
+    def JSONSCHEMANAME(self):
+        """str : Name of the schema of the data representation in JSON format."""
+        return 'polyline'
+
     @property
     def data(self):
-        """Returns the data dictionary that represents the polyline.
-
-        Returns
-        -------
-        dict
-            The polyline's data.
+        """dict : Returns the data dictionary that represents the polyline.
         """
-        return {'points': [list(point) for point in self.points]}
+        return {'points': [point.data for point in self.points]}
 
     @data.setter
     def data(self, data):
-        self.points = data['points']
+        self.points = [Point.from_data(point) for point in data['points']]
+
+    @classmethod
+    def from_data(cls, data):
+        """Construct a polyline from a data dict.
+
+        Parameters
+        ----------
+        data : dict
+            The data dictionary.
+
+        Returns
+        -------
+        :class:`compas.geometry.Polyline`
+            The constructed polyline.
+
+        Examples
+        --------
+        >>> Polyline.from_data({'points': [[0.0, 0.0, 0.0], [1.0, 0.0, 0.0], [1.0, 1.0, 0.0]]})
+        Polyline([Point(0.000, 0.000, 0.000), Point(1.000, 0.000, 0.000), Point(1.000, 1.000, 0.000)])
+
+        """
+        return cls([Point.from_data(point) for point in data['points']])
+
+    # ==========================================================================
+    # properties
+    # ==========================================================================
 
     @property
     def points(self):
-        """list of Point: The points of the polyline."""
         return self._points
 
     @points.setter
@@ -95,14 +131,12 @@ class Polyline(Primitive):
 
     @property
     def lines(self):
-        """list of :class:`compas.geometry.Line` : The lines of the polyline."""
         if not self._lines:
             self._lines = [Line(a, b) for a, b in pairwise(self.points)]
         return self._lines
 
     @property
     def length(self):
-        """float : The length of the polyline."""
         return sum([line.length for line in self.lines])
 
     # ==========================================================================
@@ -110,7 +144,7 @@ class Polyline(Primitive):
     # ==========================================================================
 
     def __repr__(self):
-        return "Polyline([{}])".format(", ".join(["{}".format(point) for point in self.points]))
+        return 'Polyline([{0}])'.format(', '.join(['{0!r}'.format(point) for point in self.points]))
 
     def __len__(self):
         return len(self.points)
@@ -134,28 +168,6 @@ class Polyline(Primitive):
     # constructors
     # ==========================================================================
 
-    @classmethod
-    def from_data(cls, data):
-        """Construct a polyline from a data dict.
-
-        Parameters
-        ----------
-        data : dict
-            The data dictionary.
-
-        Returns
-        -------
-        :class:`compas.geometry.Polyline`
-            The constructed polyline.
-
-        Examples
-        --------
-        >>> polyline = Polyline.from_data({'points': [[0.0, 0.0, 0.0], [1.0, 0.0, 0.0], [1.0, 1.0, 0.0]]})
-        >>> polyline
-        Polyline(Point(0.000, 0.000, 0.000), Point(1.000, 0.000, 0.000), Point(1.000, 1.000, 0.000))
-        """
-        return cls(data['points'])
-
     # ==========================================================================
     # methods
     # ==========================================================================
@@ -172,7 +184,7 @@ class Polyline(Primitive):
 
         Returns
         -------
-        Point
+        :class:`compas.geometry.Point`
             The point on the polyline.
 
         Examples
@@ -180,6 +192,7 @@ class Polyline(Primitive):
         >>> polyline = Polyline([[0.0, 0.0, 0.0], [1.0, 0.0, 0.0], [1.0, 1.0, 0.0]])
         >>> polyline.point(0.75)
         Point(1.000, 0.500, 0.000)
+
         """
         if t < 0 or t > 1:
             return None
@@ -220,6 +233,7 @@ class Polyline(Primitive):
         Examples
         --------
         >>>
+
         """
         raise NotImplementedError
 
@@ -239,6 +253,7 @@ class Polyline(Primitive):
         >>> polyline.points.append(polyline.points[0])
         >>> polyline.is_closed()
         True
+
         """
         return self.points[0] == self.points[-1]
 
@@ -247,7 +262,7 @@ class Polyline(Primitive):
 
         Parameters
         ----------
-        T : :class:`compas.geometry.Transformation` or list of list
+        T : :class:`compas.geometry.Transformation` | list[list[float]]
             The transformation.
 
         Examples
@@ -257,6 +272,7 @@ class Polyline(Primitive):
         >>> polyline = Polyline([[0.0, 0.0, 0.0], [1.0, 0.0, 0.0], [1.0, 1.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 0.0]])
         >>> R = Rotation.from_axis_and_angle([0.0, 0.0, 1.0], radians(90))
         >>> polyline.transform(R)
+
         """
         for index, point in enumerate(transform_points(self.points, T)):
             self.points[index].x = point[0]
@@ -266,17 +282,16 @@ class Polyline(Primitive):
     def split_at_corners(self, angle_threshold):
         """Splits a polyline at corners larger than the given angle_threshold
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         angle_threshold : float
             In radians.
 
         Returns
         -------
-        list of :class:`compas.geometry.Polyline`
+        list[:class:`compas.geometry.Polyline`]
 
         """
-
         corner_ids = []
         split_polylines = []
         points = self.points
@@ -311,54 +326,67 @@ class Polyline(Primitive):
     def tangent_at_point_on_polyline(self, point):
         """Calculates the tangent vector of a point on a polyline
 
-        Parameters:
-        -----------
-        point: :class:`compas.geometry.Point`
+        Parameters
+        ----------
+        point: [float, float, float] | :class:`compas.geometry.Point`
 
         Returns
         -------
         :class:`compas.geometry.Vector`
+
         """
         for line in self.lines:
             if is_point_on_line(point, line):
                 return line.direction
         raise Exception('{} not found!'.format(point))
 
+    tangent_at = tangent_at_point_on_polyline
+
     def divide_polyline(self, num_segments):
         """Divide a polyline in equal segments.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         num_segments : int
 
         Returns
         -------
         list
-            list of :class:`compas.geometry.Point`
+            list[:class:`compas.geometry.Point`]
+
         """
         segment_length = self.length/num_segments
-
         return self.divide_polyline_by_length(segment_length, False)
 
-    def divide_polyline_by_length(self, length, strict=True):
-        """Splits a polyline in segments of a given length
+    divide = divide_polyline
 
-        Parameters:
-        -----------
+    def divide_polyline_by_length(self, length, strict=True, tol=1e-06):
+        """Splits a polyline in segments of a given length.
+
+        Parameters
+        ----------
         length : float
-
-        strict : bool
-            If set to ``False``, the remainder segment will be added even if it is smaller than the desired length
+            Length of the segments.
+        strict : bool, optional
+            If False, the remainder segment will be added even if it is smaller than the desired length
+        tol : float, optional
+            Floating point error tolerance.
 
         Returns
         -------
-        list
-            list of :class:`compas.geometry.Point`
+        list[:class:`compas.geometry.Point`]
+
+        Notes
+        -----
+        The points of the new polyline are constrained to the segments of the old polyline.
+        However, since the old points are not part of the new set of points, the geometry of the polyline will change.
+
         """
         num_pts = int(self.length/length)
         total_length = [0, 0]
         division_pts = [self.points[0]]
         new_polyline = self
+
         for i in range(num_pts):
             for i_ln, line in enumerate(new_polyline.lines):
                 total_length.append(total_length[-1] + line.length)
@@ -367,28 +395,21 @@ class Polyline(Primitive):
                     new_pt = line.start + line.vector.scaled(amp)
                     division_pts.append(new_pt)
                     total_length = [0, 0]
-                    remaining_pts = new_polyline.points[i_ln+2:]
+                    remaining_pts = new_polyline.points[i_ln + 2:]
                     new_polyline = Polyline([new_pt, line.end] + remaining_pts)
                     break
                 elif total_length[-1] == length:
                     total_length = [0, 0]
                     division_pts.append(line.end)
 
-            if len(division_pts) == num_pts+1:
+            if len(division_pts) == num_pts + 1:
                 break
 
-        if strict is False and not self.is_closed() and len(division_pts) < num_pts+1:
+        if strict is False and not self.is_closed() and len(division_pts) < num_pts + 1:
             division_pts.append(new_polyline.points[-1])
-        elif strict is False and division_pts[-1] != self.points[-1]:
+        elif strict is False and division_pts[-1].distance_to_point(self.points[-1]) > tol:
             division_pts.append(self.points[-1])
 
         return division_pts
-# ==============================================================================
-# Main
-# ==============================================================================
 
-
-if __name__ == '__main__':
-
-    import doctest
-    doctest.testmod(globs=globals())
+    divide_by_length = divide_polyline_by_length
