@@ -12,6 +12,8 @@ Ippoliti for providing code and documentation.
 """
 import math
 
+from compas.data import Data
+
 from compas.geometry import multiply_matrices
 from compas.geometry import transpose_matrix
 
@@ -26,11 +28,8 @@ from compas.geometry.transformations import matrix_inverse
 from compas.geometry.transformations import translation_from_matrix
 
 
-__all__ = ['Transformation']
-
-
-class Transformation(object):
-    """The ``Transformation`` represents a 4x4 transformation matrix.
+class Transformation(Data):
+    """Class representing a general 4x4 transformation matrix.
 
     It is the base class for transformations like :class:`Rotation`,
     :class:`Translation`, :class:`Scale`, :class:`Reflection`,
@@ -45,9 +44,30 @@ class Transformation(object):
 
     Parameters
     ----------
-    matrix : list of list of float, optional
+    matrix : list[list[float]], optional
         The 4x4 transformation matrix.
 
+    Attributes
+    ----------
+    scale : :class:`compas.geometry.Scale`, read-only
+        The scale component of the transformation matrix.
+    shear : :class:`compas.geometry.Shear`, read-only
+        The shear component of the transformation matrix.
+    rotation : :class:`compas.geometry.Rotation`, read-only
+        The rotation component of the transformation matrix.
+    translation : :class:`compas.geometry.Translation`, read-only
+        The translation component of the transformation matrix.
+    projection : :class:`compas.geometry.Projection`, read-only
+        The projection component of the transformation matrix.
+    translation_vector : :class:`compas.geometry.Vector`, read-only
+        The translation component of the transformation matrix as a translation vector.
+    basis_vectors : tuple[:class:`compas.geometry.Vector`, :class:`compas.geometry.Vector`], read-only
+        The basis vectors from the rotation component of the transformation matrix.
+    list : list[float], read-only
+        Flattens the 4x4 transformation matrix into a list of 16 numbers.
+    determinant : float, read-only
+        The determinant of the matrix of the transformation.
+
     Examples
     --------
     >>> from compas.geometry import Frame
@@ -55,15 +75,86 @@ class Transformation(object):
     >>> T = Transformation.from_frame(f1)
     >>> Sc, Sh, R, Tl, P = T.decomposed()
     >>> Tinv = T.inverse()
+
     """
 
     def __init__(self, matrix=None):
-        """Construct a transformation from a 4x4 transformation matrix.
-        """
+        super(Transformation, self).__init__()
         if not matrix:
             matrix = identity_matrix(4)
         self.matrix = matrix
 
+    # ==========================================================================
+    # descriptors
+    # ==========================================================================
+
+    @property
+    def DATASCHEMA(self):
+        from schema import Schema
+        from compas.data import is_float4x4
+        return Schema({"matrix": is_float4x4})
+
+    @property
+    def JSONSCHEMANAME(self):
+        return 'transformation'
+
+    @property
+    def data(self):
+        return {'matrix': self.matrix}
+
+    @data.setter
+    def data(self, data):
+        self.matrix = data['matrix']
+
+    @property
+    def scale(self):
+        S, H, R, T, P = self.decomposed()
+        return S
+
+    @property
+    def shear(self):
+        S, H, R, T, P = self.decomposed()
+        return H
+
+    @property
+    def rotation(self):
+        S, H, R, T, P = self.decomposed()
+        return R
+
+    @property
+    def translation(self):
+        S, H, R, T, P = self.decomposed()
+        return T
+
+    @property
+    def projection(self):
+        S, H, R, T, P = self.decomposed()
+        return P
+
+    @property
+    def translation_vector(self):
+        from compas.geometry import Vector
+        vector = translation_from_matrix(self.matrix)
+        return Vector(*vector)
+
+    @property
+    def basis_vectors(self):
+        from compas.geometry import Vector
+        x, y = basis_vectors_from_matrix(self.rotation.matrix)
+        return Vector(*x), Vector(*y)
+
+    @property
+    def list(self):
+        return [a for c in self.matrix for a in c]
+
+    @property
+    def determinant(self):
+        return matrix_determinant(self.matrix)
+
+    # ==========================================================================
+    # customisation
+    # ==========================================================================
+
     def __mul__(self, other):
         return self.concatenated(other)
 
@@ -93,125 +184,68 @@ class Transformation(object):
         except BaseException:
             return False
 
+    def __ne__(self, other):
+        # this is not obvious to ironpython
+        return not self.__eq__(other)
+
     def __repr__(self):
-        return "Transformation({})".format(self.matrix)
+        return "Transformation({0!r})".format(self.matrix)
+
+    def __str__(self):
+        s = "[[%s],\n" % ",".join([("%.4f" % n).rjust(10) for n in self.matrix[0]])
+        s += " [%s],\n" % ",".join([("%.4f" % n).rjust(10) for n in self.matrix[1]])
+        s += " [%s],\n" % ",".join([("%.4f" % n).rjust(10) for n in self.matrix[2]])
+        s += " [%s]]\n" % ",".join([("%.4f" % n).rjust(10) for n in self.matrix[3]])
+        return s
 
     def __len__(self):
         return len(self.matrix)
 
-    def copy(self):
-        """Returns a copy of the transformation.
-        """
-        cls = type(self)
-        matrix = [
-            self.matrix[0][:],
-            self.matrix[1][:],
-            self.matrix[2][:],
-            self.matrix[3][:]]
-        return cls(matrix)
-
-    @property
-    def data(self):
-        """Return a ``Transformation`` object's to a data dict.
-
-        Returns
-        -------
-        dict
-            A dictionary with a transformation matrix stored under the key "matrix".
-
-        Examples
-        --------
-        >>> matrix = [[1, 0, 0, 3], [0, 1, 0, 4], [0, 0, 1, 5], [0, 0, 0, 1]]
-        >>> data = {'matrix': matrix}
-        >>> T = Transformation.from_data(data)
-        >>> T.data == data
-        True
-        """
-        return {'matrix': self.matrix}
-
-    @data.setter
-    def data(self, data):
-        self.matrix = data['matrix']
-
-    @classmethod
-    def from_data(cls, data):
-        """Creates a ``Transformation`` from a data dict.
-
-        Parameters
-        ----------
-        data : :obj:`dict`
-            A dictionary with a transformation matrix stored under the key "matrix".
-
-        Returns
-        -------
-        Transformation
-            The ``Transformation`` object.
-
-        Examples
-        --------
-        >>> matrix = [[1, 0, 0, 3], [0, 1, 0, 4], [0, 0, 1, 5], [0, 0, 0, 1]]
-        >>> data = {'matrix': matrix}
-        >>> T = Transformation.from_data(data)
-        """
-        return cls(data['matrix'])
-
-    def to_data(self):
-        """Convert a ``Transformation`` object to a data dict.
-
-        Returns
-        -------
-        dict
-            A dictionary with a transformation matrix stored under the key "matrix".
-
-        Examples
-        --------
-        >>> matrix = [[1, 0, 0, 3], [0, 1, 0, 4], [0, 0, 1, 5], [0, 0, 0, 1]]
-        >>> data = {'matrix': matrix}
-        >>> T = Transformation.from_data(data)
-        >>> T.to_data() == data
-        True
-        """
-        return self.data
+    # ==========================================================================
+    # constructors
+    # ==========================================================================
 
     @classmethod
     def from_matrix(cls, matrix):
-        """Creates a ``Transformation`` from a 4x4 matrix-like object.
+        """Creates a transformation from a list[list[float]] object.
 
         Parameters
         ----------
-        matrix : 4x4 matrix-like
+        matrix : list[list[float]]
             The 4x4 transformation matrix.
 
         Returns
         -------
-        Transformation
-            The ``Transformation`` object.
+        :class:`compas.geometry.Transformation`
+            The transformation.
+
         """
         return cls(matrix)
 
     @classmethod
     def from_list(cls, numbers):
-        """Creates a ``Transformation`` from a list of 16 numbers.
+        """Creates a transformation from a list of 16 numbers.
 
         Parameters
         ----------
-        numbers : :obj:`list` of :obj:`float`
+        numbers : list[float]
             A list of 16 numbers
 
         Returns
         -------
-        Transformation
-            The ``Transformation`` object.
+        :class:`compas.geometry.Transformation`
+            The transformation.
+
+        Notes
+        -----
+        Since the transformation matrix follows the row-major order, the
+        translational components must be at the list's indices 3, 7, 11.
 
         Examples
         --------
         >>> numbers = [1, 0, 0, 3, 0, 1, 0, 4, 0, 0, 1, 5, 0, 0, 0, 1]
         >>> T = Transformation.from_list(numbers)
 
-        Notes
-        -----
-        Since the transformation matrix follows the row-major order, the
-        translational components must be at the list's indices 3, 7, 11.
         """
         matrix = identity_matrix(4)
         for i in range(4):
@@ -220,29 +254,26 @@ class Transformation(object):
         return cls(matrix)
 
     @classmethod
-    def from_euler_angles(cls, euler_angles, static=True,
-                          axes='xyz', point=[0, 0, 0]):
+    def from_euler_angles(cls, euler_angles, static=True, axes='xyz', point=[0, 0, 0]):
         """Construct a transformation from a rotation represented by Euler angles.
 
         Parameters
         ----------
-        euler_angles : list of float
+        euler_angles : [float, float, float]
             Three numbers that represent the angles of rotations about the defined axes.
         static : bool, optional
-            If true the rotations are applied to a static frame.
-            If not, to a rotational.
-            Defaults to ``True``.
+            If True the rotations are applied to a static frame.
+            If False, to a rotational.
         axes : str, optional
             A 3 character string specifying the order of the axes.
-            Defaults to ``'xyz'``.
-        point : list of float, optional
+        point : list[float], optional
             The point of the frame.
-            Defaults to ``[0, 0, 0]``.
 
         Returns
         -------
         :class:`compas.geometry.Transformation`
-            The constructed transformation.
+            The transformation.
+
         """
         R = matrix_from_euler_angles(euler_angles, static, axes)
         T = matrix_from_translation(point)
@@ -252,17 +283,21 @@ class Transformation(object):
     # should not one of the two just have a "to" function
     @classmethod
     def from_frame(cls, frame):
-        """Computes a transformation from world XY to frame.
+        """Construct a transformation from world XY to frame.
 
         Parameters
         ----------
-        frame : :class:`Frame`
+        frame : :class:`compas.geometry.Frame`
             A frame describing the targeted Cartesian coordinate system.
 
         Returns
         -------
-        Transformation
-            The ``Transformation`` object.
+        :class:`compas.geometry.Transformation`
+            The transformation.
+
+        Notes
+        -----
+        It is the same as from_frame_to_frame(Frame.worldXY(), frame).
 
         Examples
         --------
@@ -273,32 +308,28 @@ class Transformation(object):
         >>> f1 == f2
         True
 
-        Notes
-        -----
-        It is the same as from_frame_to_frame(Frame.worldXY(), frame).
-
         """
         return cls(matrix_from_frame(frame))
 
     @classmethod
     def from_frame_to_frame(cls, frame_from, frame_to):
-        """Computes a transformation between two frames.
+        """Construct a transformation between two frames.
 
         This transformation allows to transform geometry from one Cartesian
-        coordinate system defined by "frame_from" to another Cartesian
-        coordinate system defined by "frame_to".
+        coordinate system defined by `frame_from` to another Cartesian
+        coordinate system defined by `frame_to`.
 
         Parameters
         ----------
-        frame_from : :class:`Frame`
+        frame_from : :class:`compas.geometry.Frame`
             A frame defining the original Cartesian coordinate system.
-        frame_to : :class:`Frame`
+        frame_to : :class:`compas.geometry.Frame`
             A frame defining the targeted Cartesian coordinate system.
 
         Returns
         -------
-        Transformation
-            The ``Transformation`` object representing a change of basis.
+        :class:`compas.geometry.Transformation`
+            The transformation.
 
         Examples
         --------
@@ -309,6 +340,7 @@ class Transformation(object):
         >>> f1.transform(T)
         >>> f1 == f2
         True
+
         """
         T1 = cls.from_frame(frame_from)
         T2 = cls.from_frame(frame_to)
@@ -316,18 +348,23 @@ class Transformation(object):
 
     @classmethod
     def from_change_of_basis(cls, frame_from, frame_to):
-        """Computes a change of basis transformation between two frames.
+        """Construct a change of basis transformation between two frames.
 
         A basis change is essentially a remapping of geometry from one
         coordinate system to another.
 
         Parameters
         ----------
-        frame_from : :class:`Frame`
+        frame_from : :class:`compas.geometry.Frame`
             A frame defining the original Cartesian coordinate system.
-        frame_to : :class:`Frame`
+        frame_to : :class:`compas.geometry.Frame`
             A frame defining the targeted Cartesian coordinate system.
 
+        Returns
+        -------
+        :class:`compas.geometry.Transformation`
+            The transformation representing a change of basis.
+
         Examples
         --------
         >>> from compas.geometry import Point, Frame
@@ -339,111 +376,32 @@ class Transformation(object):
         Point(1.395, 0.955, 1.934)
         >>> Frame.local_to_local_coordinates(f1, f2, p_f1)
         Point(1.395, 0.955, 1.934)
+
         """
         T1 = cls.from_frame(frame_from)
         T2 = cls.from_frame(frame_to)
         return cls(multiply_matrices(matrix_inverse(T2.matrix), T1.matrix))
 
-    @property
-    def scale(self):
-        """The scale component of the transformation matrix.
+    # ==========================================================================
+    # methods
+    # ==========================================================================
+
+    def copy(self):
+        """Returns a copy of the transformation.
 
         Returns
         -------
-        compas.geometry.Scale
-            The scale component of the transformation.
+        :class:`compas.geometry.Transformation`
+            An independent copy of the transformation.
+
         """
-        S, H, R, T, P = self.decomposed()
-        return S
-
-    @property
-    def shear(self):
-        """The shear component of the transformation matrix.
-
-        Returns
-        -------
-        compas.geometry.Shear
-            The shear component of the transformation.
-        """
-        S, H, R, T, P = self.decomposed()
-        return H
-
-    @property
-    def rotation(self):
-        """The rotation component of the transformation matrix.
-
-        Returns
-        -------
-        compas.geometry.Rotation
-            The rotation component of the transformation.
-        """
-        S, H, R, T, P = self.decomposed()
-        return R
-
-    @property
-    def translation(self):
-        """The translation component of the transformation matrix.
-
-        Returns
-        -------
-        compas.geometry.Translation
-            The translation component of the transformation.
-        """
-        S, H, R, T, P = self.decomposed()
-        return T
-
-    @property
-    def projection(self):
-        """The projection component of the transformation matrix.
-
-        Returns
-        -------
-        compas.geometry.Projection
-            The projectionn component of the transformation.
-        """
-        S, H, R, T, P = self.decomposed()
-        return P
-
-    @property
-    def translation_vector(self):
-        from compas.geometry import Vector
-        vector = translation_from_matrix(self.matrix)
-        return Vector(*vector)
-
-    @property
-    def basis_vectors(self):
-        """The basis vectors from the rotation component of the transformation matrix.
-
-        Returns
-        -------
-        tuple of :class:`compas.geometry.Vector`
-            The basis vectors of the rotation component of the tranformation.
-        """
-        from compas.geometry import Vector
-        x, y = basis_vectors_from_matrix(self.rotation.matrix)
-        return Vector(*x), Vector(*y)
-
-    @property
-    def list(self):
-        """Flattens the 4x4 transformation matrix into a list of 16 numbers.
-
-        Returns
-        -------
-        list
-            The transformation matrix as a flattened list in row-major order.
-        """
-        return [a for c in self.matrix for a in c]
-
-    @property
-    def determinant(self):
-        """The determinant of the matrix of the transformation.
-
-        Returns
-        -------
-        float
-            The determinant of the matrix of this transformation.
-        """
-        return matrix_determinant(self.matrix)
+        cls = type(self)
+        matrix = [
+            self.matrix[0][:],
+            self.matrix[1][:],
+            self.matrix[2][:],
+            self.matrix[3][:]]
+        return cls(matrix)
 
     def transpose(self):
         """Transpose the matrix of this transformation.
@@ -452,6 +410,7 @@ class Transformation(object):
         -------
         None
             The transformation is transposed in-place.
+
         """
         self.matrix = transpose_matrix(self.matrix)
 
@@ -460,15 +419,23 @@ class Transformation(object):
 
         Returns
         -------
-        Transformation
+        :class:`compas.geometry.Transformation`
             The transposed transformation object.
+
         """
         T = self.copy()
         T.transpose()
         return T
 
     def invert(self):
-        """Invert this transformation."""
+        """Invert this transformation.
+
+        Returns
+        -------
+        None
+            The transformation is transposed in-place.
+
+        """
         self.matrix = matrix_inverse(self.matrix)
 
     def inverse(self):
@@ -476,7 +443,7 @@ class Transformation(object):
 
         Returns
         -------
-        Transformation
+        :class:`compas.geometry.Transformation`
             The inverse transformation.
 
         Examples
@@ -487,6 +454,7 @@ class Transformation(object):
         >>> I = Transformation(identity_matrix(4))
         >>> I == T * T.inverse()
         True
+
         """
         T = self.copy()
         T.invert()
@@ -495,17 +463,24 @@ class Transformation(object):
     inverted = inverse
 
     def decomposed(self):
-        """Decompose the ``Transformation`` into its ``Scale``, ``Shear``,
-        ``Rotation``, ``Translation`` and ``Projection`` components.
+        """Decompose the `Transformation` into its components.
 
         Returns
         -------
-        5-tuple of Transformation
-            The scale, shear, rotation, translation, and projection components
-            of the current transformation.
+        :class:`compas.geometry.Scale`
+            The scale component of the current transformation.
+        :class:`compas.geometry.Shear`
+            The shear component of the current transformation.
+        :class:`compas.geometry.Rotation`
+            The rotation component of the current transformation.
+        :class:`compas.geometry.Translation`
+            The translation component of the current transformation.
+        :class:`compas.geometry.Projection`
+            The projection component of the current transformation.
 
         Examples
         --------
+        >>> from compas.geometry import Scale, Translation, Rotation
         >>> trans1 = [1, 2, 3]
         >>> angle1 = [-2.142, 1.141, -0.142]
         >>> scale1 = [0.123, 2, 0.5]
@@ -520,6 +495,7 @@ class Transformation(object):
         True
         >>> T1 == T
         True
+
         """
         from compas.geometry import Scale  # noqa: F811
         from compas.geometry import Shear
@@ -550,11 +526,12 @@ class Transformation(object):
         Notes
         -----
         Rz * Ry * Rx means that Rx is first transformation, Ry second, and Rz third.
+
         """
         self.matrix = multiply_matrices(self.matrix, other.matrix)
 
     def concatenated(self, other):
-        """Concatenate two transformations into one ``Transformation``.
+        """Concatenate two transformations into one `Transformation`.
 
         Parameters
         ----------
@@ -563,60 +540,15 @@ class Transformation(object):
 
         Returns
         -------
-        T : :class:`compas.geometry.Transformation`
+        :class:`compas.geometry.Transformation`
             The new transformation that is the concatenation of this one and the other.
 
         Notes
         -----
         Rz * Ry * Rx means that Rx is first transformation, Ry second, and Rz third.
+
         """
         cls = type(self)
         if isinstance(other, cls):
             return cls(multiply_matrices(self.matrix, other.matrix))
         return Transformation(multiply_matrices(self.matrix, other.matrix))
-
-
-# ==============================================================================
-# Main
-# ==============================================================================
-
-if __name__ == "__main__":
-
-    from compas.geometry import Translation  # noqa: F401
-    from compas.geometry import Rotation  # noqa: F401
-    from compas.geometry import Scale  # noqa: F401
-    from compas.geometry import Frame  # noqa: F401
-
-    import doctest
-    doctest.testmod(globs=globals())
-
-    # world = Frame.worldXY()
-    # frame = Frame([1.0, 1.0, 1.0], [0, 0, -1], [1, 0, 0])
-
-    # X1 = Transformation.from_frame_to_frame(world, frame)
-    # X2 = Transformation.from_frame(frame)
-    # X3 = Transformation.from_change_of_basis(frame, world)
-
-    # print(X1.matrix)
-    # print(X2.matrix)
-    # print(X3.matrix)
-
-    # trans1 = [1, 2, 3]
-    # angle1 = [-2.142, 1.141, -0.142]
-    # scale1 = [0.123, 2, 0.5]
-    # T1 = Translation.from_vector(trans1)
-    # R1 = Rotation.from_euler_angles(angle1)
-    # S1 = Scale.from_factors(scale1)
-    # M = T1 * R1 * S1
-    # S, H, R, T, P = M.decomposed()
-    # print(S1 == S)
-    # print(R1 == R)
-    # print(T1 == T)
-
-    # S, H, R, T, P = X3.decomposed()
-
-    # print(S)
-    # print(H)
-    # print(R)
-    # print(T)
-    # print(P)
