@@ -5,6 +5,12 @@ artists
 
 .. currentmodule:: compas_ghpython.artists
 
+.. rst-class:: lead
+
+Artists for visualizing (painting) COMPAS geometry, robots, and data structures in Grasshopper using the GH Python component.
+The artists in this package provide plugins for the pluggable methods of the base artists in :mod:`compas.artists`.
+Therefore, they can be used directly, from here, or through the base artists :class:`compas.artists.Artist`.
+
 
 Primitive Artists
 =================
@@ -17,6 +23,7 @@ Primitive Artists
     CapsuleArtist
     CircleArtist
     ConeArtist
+    CurveArtist
     CylinderArtist
     FrameArtist
     LineArtist
@@ -25,6 +32,7 @@ Primitive Artists
     PolyhedronArtist
     PolylineArtist
     SphereArtist
+    SurfaceArtist
     TorusArtist
     VectorArtist
 
@@ -71,6 +79,7 @@ from compas.geometry import Box
 from compas.geometry import Capsule
 from compas.geometry import Circle
 from compas.geometry import Cone
+from compas.geometry import Curve
 from compas.geometry import Cylinder
 from compas.geometry import Frame
 from compas.geometry import Line
@@ -79,6 +88,7 @@ from compas.geometry import Polygon
 from compas.geometry import Polyhedron
 from compas.geometry import Polyline
 from compas.geometry import Sphere
+from compas.geometry import Surface
 from compas.geometry import Torus
 from compas.geometry import Vector
 
@@ -93,6 +103,7 @@ from .boxartist import BoxArtist
 from .capsuleartist import CapsuleArtist
 from .circleartist import CircleArtist
 from .coneartist import ConeArtist
+from .curveartist import CurveArtist
 from .cylinderartist import CylinderArtist
 from .frameartist import FrameArtist
 from .lineartist import LineArtist
@@ -104,6 +115,7 @@ from .polyhedronartist import PolyhedronArtist
 from .polylineartist import PolylineArtist
 from .robotmodelartist import RobotModelArtist
 from .sphereartist import SphereArtist
+from .surfaceartist import SurfaceArtist
 from .torusartist import TorusArtist
 from .vectorartist import VectorArtist
 from .volmeshartist import VolMeshArtist
@@ -132,6 +144,7 @@ def register_artists():
     Artist.register(Capsule, CapsuleArtist, context='Grasshopper')
     Artist.register(Circle, CircleArtist, context='Grasshopper')
     Artist.register(Cone, ConeArtist, context='Grasshopper')
+    Artist.register(Curve, CurveArtist, context='Grasshopper')
     Artist.register(Cylinder, CylinderArtist, context='Grasshopper')
     Artist.register(Frame, FrameArtist, context='Grasshopper')
     Artist.register(Line, LineArtist, context='Grasshopper')
@@ -143,6 +156,7 @@ def register_artists():
     Artist.register(Polyline, PolylineArtist, context='Grasshopper')
     Artist.register(RobotModel, RobotModelArtist, context='Grasshopper')
     Artist.register(Sphere, SphereArtist, context='Grasshopper')
+    Artist.register(Surface, SurfaceArtist, context='Grasshopper')
     Artist.register(Torus, TorusArtist, context='Grasshopper')
     Artist.register(Vector, VectorArtist, context='Grasshopper')
     Artist.register(VolMesh, VolMeshArtist, context='Grasshopper')
@@ -156,6 +170,7 @@ __all__ = [
     'CapsuleArtist',
     'CircleArtist',
     'ConeArtist',
+    'CurveArtist',
     'CylinderArtist',
     'FrameArtist',
     'LineArtist',
@@ -167,6 +182,7 @@ __all__ = [
     'PolylineArtist',
     'RobotModelArtist',
     'SphereArtist',
+    'SurfaceArtist',
     'TorusArtist',
     'VectorArtist',
     'VolMeshArtist',
