@@ -27,6 +27,7 @@ __all__ = [
     'get_object_names',
     'get_object_name',
     'get_object_attributes',
+    'set_object_attributes',
     'get_object_attributes_from_name',
     'delete_object',
     'delete_objects',
@@ -81,16 +82,18 @@ def get_objects(name=None, color=None, layer=None, type=None):
     Parameters
     ----------
     name : str, optional
+        Name of the objects.
     color : tuple or list, optional
         RGB color components in integer format (0-255).
     layer : str, optional
+        Layer containing the objects.
     type : Rhino.DocObjects.ObjectType, optional
         The object type.
 
     Returns
     -------
-    list
-        The GUIDs of the objects matching the filter parameters.
+    list[System.Guid]
+        The System.Guids of the objects matching the filter parameters.
 
     Examples
     --------
@@ -142,58 +145,84 @@ def get_objects(name=None, color=None, layer=None, type=None):
     return guids
 
 
-def delete_object(guid, purge=None):
+def delete_object(guid, purge=None, redraw=True):
     """Delete Rhino object.
 
     Parameters
     ----------
-    guid : GUID
+    guid : System.Guid
+        Object identifier.
     purge : None or bool, optional
         If None, the value of the global purge setting (:obj:`compas_rhino.PURGE_ON_DELETE`) will be used.
         If True, purge the object from history after deleting.
         If False, delete but don't purge.
-        Default is None.
+    redraw : bool, optional
+        If True, redrawing will be enabled and enacted.
+        If False, redrawing will be disabled.
+
+    Returns
+    -------
+    None
+
     """
     if purge is None:
         purge = compas_rhino.PURGE_ON_DELETE
     if purge and purge_object:
-        purge_objects([guid])
+        purge_objects([guid], redraw=redraw)
     else:
-        delete_objects([guid], purge)
+        delete_objects([guid], purge, redraw=redraw)
 
 
-def delete_objects(guids, purge=None):
+def delete_objects(guids, purge=None, redraw=True):
     """Delete multiple Rhino objects.
 
     Parameters
     ----------
-    guids : list of GUID
+    guids : list[System.Guid]
+        Object identifiers.
     purge : None or bool, optional
         If None, the value of the global purge setting (:obj:`compas_rhino.PURGE_ON_DELETE`) will be used.
         If True, purge the objects from history after deleting.
         If False, delete but don't purge.
-        Default is None.
+    redraw : bool, optional
+        If True, redrawing will be enabled and enacted.
+        If False, redrawing will be disabled.
+
+    Returns
+    -------
+    None
+
     """
     if purge is None:
         purge = compas_rhino.PURGE_ON_DELETE
     if purge and purge_object:
-        purge_objects(guids)
+        purge_objects(guids, redraw=redraw)
     else:
         rs.EnableRedraw(False)
         for guid in guids:
             if rs.IsObjectHidden(guid):
                 rs.ShowObject(guid)
         rs.DeleteObjects(guids)
-        rs.EnableRedraw(True)
-        sc.doc.Views.Redraw()
+        if redraw:
+            rs.EnableRedraw(True)
+            sc.doc.Views.Redraw()
 
 
-def purge_objects(guids):
+def purge_objects(guids, redraw=True):
     """Purge objects from memory.
 
     Parameters
     ----------
-    guids : list of GUID
+    guids : list[System.Guid]
+        Object identifiers.
+    redraw : bool, optional
+        If True, redrawing will be enabled and enacted.
+        If False, redrawing will be disabled.
+
+    Returns
+    -------
+    None
+
     """
     if not purge_object:
         raise RuntimeError('Cannot purge outside Rhino script context')
@@ -204,8 +233,9 @@ def purge_objects(guids):
                 rs.ShowObject(guid)
             o = find_object(guid)
             purge_object(o.RuntimeSerialNumber)
-    rs.EnableRedraw(True)
-    sc.doc.Views.Redraw()
+    if redraw:
+        rs.EnableRedraw(True)
+        sc.doc.Views.Redraw()
 
 
 def get_object_layers(guids):
@@ -213,48 +243,114 @@ def get_object_layers(guids):
 
     Parameters
     ----------
-    guids : list of GUID
+    guids : list[System.Guid]
+        Object identifiers.
 
     Returns
     -------
+    list[str]
 
     """
     return [rs.ObjectLayer(guid) for guid in guids]
 
 
 def get_object_types(guids):
+    """Get the type of multiple objects.
+
+    Parameters
+    ----------
+    guids : list[System.Guid]
+        Object identifiers.
+
+    Returns
+    -------
+    list[str]
+
+    """
     return [rs.ObjectType(guid) for guid in guids]
 
 
 def get_object_names(guids):
+    """Get the names of multiple objects.
+
+    Parameters
+    ----------
+    guids : list[System.Guid]
+        Object identifiers.
+
+    Returns
+    -------
+    list[str]
+
+    """
     return [rs.ObjectName(guid) for guid in guids]
 
 
 def get_object_name(guid):
-    return rs.ObjectName(guid)
-
-
-def get_object_attributes(guids):
-    """Get attributes from object user dictionaries.
+    """Get the name of one object.
 
     Parameters
     ----------
-    guids : list of GUID
+    guid : System.Guid
+        Object identifier.
 
     Returns
     -------
-    list of dict
+    str
+
     """
-    attrs = []
-    for guid in guids:
-        o = find_object(guid)
-        u = o.Attributes.UserDictionary
-        a = {}
-        if u.Count:
-            for key in u.Keys:
-                a[key] = u.Item[key]
-        attrs.append(a)
-    return attrs
+    return rs.ObjectName(guid)
+
+
+def get_object_attributes(guid):
+    """Get attributes from an object's user dictionary.
+
+    Parameters
+    ----------
+    guid : System.Guid
+        Object identifier.
+
+    Returns
+    -------
+    dict[str, Any]
+
+    """
+    o = find_object(guid)
+    u = o.Attributes.UserDictionary
+    a = {}
+    if u.Count:
+        for key in u.Keys:
+            a[key] = u.Item[key]
+    return a
+
+
+def set_object_attributes(guid, attr):
+    """Set the custom attributes of a Rhino object.
+
+    Parameters
+    ----------
+    guid : System.Guid
+        Object identifier.
+    attr : dict[str, Any]
+        A dictionary of attributes.
+
+    Returns
+    -------
+    None
+
+    Raises
+    ------
+    Exception
+        If the operation fails.
+
+    """
+    o = find_object(guid)
+    u = o.Attributes.UserDictionary
+    for name, value in iter(attr.items()):
+        try:
+            u.Set(name, value)
+        except Exception:
+            print("The following item cannot be stored in the user dictionary of this object: {0} => {1}".format(name, value))
 
 
 def get_object_attributes_from_name(guids, prefix=None):
@@ -262,7 +358,8 @@ def get_object_attributes_from_name(guids, prefix=None):
 
     Parameters
     ----------
-    guids : list of GUID
+    guids : list[System.Guid]
+        Object identifiers.
     prefix : str, optional
         A prefix that should be removed before the name is JSON parsable.
         For example, in Rhino 6 and above, names can't start with curly braces ("{").
@@ -271,8 +368,9 @@ def get_object_attributes_from_name(guids, prefix=None):
 
     Results
     -------
-    list of dict
+    list[dict[str, Any]]
         The attribute dicts of the objects.
+
     """
     load = json.loads
     attrs = []
@@ -293,12 +391,13 @@ def select_object(message="Select an object."):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an object.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected object.
+
     """
     return rs.GetObject(message)
 
@@ -309,12 +408,13 @@ def select_objects(message='Select multiple objects.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select objects.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected objects.
+
     """
     guids = []
     temp = rs.GetObjects(message)
@@ -334,12 +434,13 @@ def select_point(message='Select one point.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an point.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected point.
+
     """
     return rs.GetObject(message, preselect=True, select=True, filter=rs.filter.point)
 
@@ -350,12 +451,13 @@ def select_points(message='Select multiple points.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select points.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected points.
+
     """
     guids = []
     temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.point)
@@ -365,6 +467,19 @@ def select_points(message='Select multiple points.'):
 
 
 def get_points(layer=None):
+    """Get all points.
+
+    Parameters
+    ----------
+    layer : str, optional
+        Name of a layer containing the points.
+
+    Returns
+    -------
+    list[System.Guid]
+        The identifiers of the points.
+
+    """
     if layer:
         rs.EnableRedraw(False)
         # Argument names for LayerVisible command are not the same for Rhino5 and Rhino6
@@ -384,12 +499,13 @@ def get_point_coordinates(guids):
 
     Parameters
     ----------
-    guids : list of GUID
+    guids : list[System.Guid]
 
     Returns
     -------
-    list of point
+    list[[float, float, float]]
         The location coordinates of the points.
+
     """
     points = []
     for guid in guids:
@@ -405,6 +521,20 @@ def get_point_coordinates(guids):
 
 
 def is_curve_line(guid):
+    """Verify that a curve is a line.
+
+    Parameters
+    ----------
+    guid : System.Guid
+        The identifier of the curve.
+
+    Returns
+    -------
+    bool
+        True if the curve is a line.
+        False otherwise.
+
+    """
     return rs.IsCurve(guid) and rs.IsLine(guid) and rs.CurveDegree(guid) == 1 and len(rs.CurvePoints(guid)) == 2
 
 
@@ -422,12 +552,13 @@ def select_curve(message='Select one curve.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an line.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected curve.
+
     """
     return rs.GetObject(message, preselect=True, select=True, filter=rs.filter.curve)
 
@@ -438,12 +569,13 @@ def select_curves(message='Select multiple curves.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an line.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected curves.
+
     """
     guids = []
     temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
@@ -458,12 +590,13 @@ def select_line(message='Select line.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an line.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected line.
+
     """
     guid = rs.GetObject(message, preselect=True, select=True, filter=rs.filter.curve)
     if is_curve_line(guid):
@@ -477,12 +610,13 @@ def select_lines(message='Select multiple lines.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select lines.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected lines.
+
     """
     guids = []
     temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
@@ -499,12 +633,13 @@ def select_polyline(message='Select one polyline (curve with degree = 1, and mul
     Parameters
     ----------
     message : str, optional
-        Default is "Select an polyline.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected polyline.
+
     """
     guid = rs.GetObject(message, preselect=True, select=True, filter=rs.filter.curve)
     if is_curve_polyline(guid):
@@ -518,12 +653,13 @@ def select_polylines(message='Select multiple polylines (curves with degree = 1,
     Parameters
     ----------
     message : str, optional
-        Default is "Select polylines.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected polylines.
+
     """
     guids = []
     temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
@@ -540,12 +676,13 @@ def select_polygon(message='Select one polygon (closed curve with degree = 1)'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an polygon.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected polygon.
+
     """
     guid = rs.GetObject(message, preselect=True, select=True, filter=rs.filter.curve)
     if is_curve_polygon(guid):
@@ -559,12 +696,13 @@ def select_polygons(message='Select multiple polygons (closed curves with degree
     Parameters
     ----------
     message : str, optional
-        Default is "Select polygons.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected polygons.
+
     """
     guids = []
     temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.curve)
@@ -576,6 +714,19 @@ def select_polygons(message='Select multiple polygons (closed curves with degree
 
 
 def get_curves(layer=None):
+    """Get all curves.
+
+    Parameters
+    ----------
+    layer : str, optional
+        Name of a layer containing the curves.
+
+    Returns
+    -------
+    list[System.Guid]
+        The identifiers of the curves.
+
+    """
     if layer:
         rs.EnableRedraw(False)
         # Argument names for LayerVisible command are not the same for Rhino5 and Rhino6
@@ -591,6 +742,19 @@ def get_curves(layer=None):
 
 
 def get_lines(layer=None):
+    """Get all lines.
+
+    Parameters
+    ----------
+    layer : str, optional
+        Name of a layer containing the lines.
+
+    Returns
+    -------
+    list[System.Guid]
+        The identifiers of the lines.
+
+    """
     if layer:
         rs.EnableRedraw(False)
         # Argument names for LayerVisible command are not the same for Rhino5 and Rhino6
@@ -608,6 +772,19 @@ def get_lines(layer=None):
 
 
 def get_polylines(layer=None):
+    """Get all polylines.
+
+    Parameters
+    ----------
+    layer : str, optional
+        Name of a layer containing the polylines.
+
+    Returns
+    -------
+    list[System.Guid]
+        The identifiers of the polylines.
+
+    """
     if layer:
         rs.EnableRedraw(False)
         # Argument names for LayerVisible command are not the same for Rhino5 and Rhino6
@@ -625,6 +802,19 @@ def get_polylines(layer=None):
 
 
 def get_polygons(layer=None):
+    """Get all polygons.
+
+    Parameters
+    ----------
+    layer : str, optional
+        Name of a layer containing the polygons.
+
+    Returns
+    -------
+    list[System.Guid]
+        The identifiers of the polygons.
+
+    """
     if layer:
         rs.EnableRedraw(False)
         # Argument names for LayerVisible command are not the same for Rhino5 and Rhino6
@@ -641,11 +831,20 @@ def get_polygons(layer=None):
     return guids
 
 
-def get_curve_coordinates():
-    pass
-
-
 def get_line_coordinates(guids):
+    """Get the start and end point coordinates of line curves.
+
+    Parameters
+    ----------
+    guids : list[System.Guid]
+        Line curve identifiers.
+
+    Returns
+    -------
+    list[tuple[[float, float, float], [float, float, float]]]
+        A start and end point per line.
+
+    """
     if isinstance(guids, System.Guid):
         sp = map(float, rs.CurveStartPoint(guids))
         ep = map(float, rs.CurveEndPoint(guids))
@@ -658,11 +857,20 @@ def get_line_coordinates(guids):
     return lines
 
 
-def get_polycurve_coordinates():
-    pass
-
-
 def get_polyline_coordinates(guids):
+    """Get the point coordinates of polylines.
+
+    Parameters
+    ----------
+    guids : list[System.Guid]
+        Polyline curve identifiers.
+
+    Returns
+    -------
+    list[list[[float, float, float]]]
+        A list of point coordinates per polyline.
+
+    """
     if isinstance(guids, System.Guid):
         points = rs.PolylineVertices(guids)
         coords = []
@@ -680,6 +888,19 @@ def get_polyline_coordinates(guids):
 
 
 def get_polygon_coordinates(guids):
+    """Get the point coordinates of polygons.
+
+    Parameters
+    ----------
+    guids : list[System.Guid]
+        Polygon curve identifiers.
+
+    Returns
+    -------
+    list[list[[float, float, float]]]
+        A list of point coordinates per polygon.
+
+    """
     if isinstance(guids, System.Guid):
         points = rs.CurvePoints(guids)
         coords = []
@@ -708,12 +929,13 @@ def select_surface(message='Select one surface.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an surface.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected surface.
+
     """
     return rs.GetObject(
         message, preselect=True, select=True,
@@ -726,12 +948,13 @@ def select_surfaces(message='Select multiple surfaces.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select surfaces.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected surfaces.
+
     """
     guids = []
     temp = rs.GetObjects(
@@ -753,12 +976,13 @@ def select_mesh(message='Select one mesh.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select an mesh.".
+        Instruction for the user.
 
     Returns
     -------
-    GUID
+    System.Guid
         The identifer of the selected mesh.
+
     """
     return rs.GetObject(
         message, preselect=True, select=True,
@@ -772,12 +996,13 @@ def select_meshes(message='Select multiple meshes.'):
     Parameters
     ----------
     message : str, optional
-        Default is "Select meshs.".
+        Instruction for the user.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
         The identifers of the selected meshs.
+
     """
     guids = []
     temp = rs.GetObjects(message, preselect=True, select=True, group=False, filter=rs.filter.mesh)
@@ -955,11 +1180,3 @@ def get_mesh_edge_vertex_indices(guid):
         temp = mgeo.TopologyVertices.MeshVertexIndices(tvindex)
         vindices.append(temp[0])
     return vindices
-
-
-# ==============================================================================
-# Main
-# ==============================================================================
-
-if __name__ == '__main__':
-    pass
